@@ -20,7 +20,7 @@ use Clone qw(clone);
 use Data::Dumper;
 use JSON;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 has url => (
     is => 'ro' ,
@@ -43,10 +43,10 @@ has sn => (
     }
 );
 
-has pattern => (
+has query_pattern => (
     is      => 'ro',
     lazy    => 1,
-    builder => 'get_pattern'
+    builder => 'get_query_pattern'
 );
 
 has lru => (
@@ -65,32 +65,52 @@ has log => (
 	}
 );
 
+# Public method
 sub is_fragment_server {
-    shift->pattern ? 1 : 0;
+    shift->query_pattern ? 1 : 0;
 }
 
-sub get_sparql {
-    my ($self,$sparql) = @_;
+# Public method
+# Optimized method to find all bindings mayching a pattern
+# See:
+# Verborgh, Ruden, et al. Querying Datasets on the Web with High Availability. ISWC2014
+# http://linkeddatafragments.org/publications/iswc2014.pdf
+sub get_pattern {
+    my ($self,$bgp,$context,%args) = @_;
 
-    $self->log->debug("parsing $sparql");
-    my $query = RDF::Query->new($sparql);
+    my (@triples)   = ($bgp->isa('RDF::Trine::Statement') or $bgp->isa('RDF::Query::Algebra::Filter'))
+                    ? $bgp
+                    : $bgp->triples;
 
-    unless (defined $query) {
-        $self->log->error("failed to parse $sparql");
-        return undef;
+    unless (@triples) {
+        die "can't execute get_pattern for an empty pattern";
     }
 
-    my $bgps = $self->_parse_bgp($query->pattern);
+    my @vars = $bgp->referenced_variables;
 
-    unless (is_array_ref($bgps)) {
-        $self->log->error("can't parse bgp for sparql");
-        return undef;
-    }
+    my @bgps = map { $self->_parse_triple_pattern($_)} @triples;
+    
+    my $sub = sub {
+        state $it = $self->_find_variable_bindings(\@bgps);
+        my $b = $it->();
 
-    $self->find_variable_bindings($bgps);
+        return undef unless $b;
+
+        my $binding = RDF::Trine::VariableBindings->new({});
+
+        for my $key (keys %$b) {
+            my $val = $b->{$key};
+            $key =~ s{^\?}{};
+            $binding->set($key => $val);
+        }
+
+        $binding;
+    };
+
+    RDF::Trine::Iterator::Bindings->new($sub,\@vars);
 }
 
-sub find_variable_bindings {
+sub _find_variable_bindings {
     my $self     = shift;
     my $bgps     = shift;
     my $bindings = shift // {};
@@ -105,7 +125,7 @@ sub find_variable_bindings {
         while (!defined($ret = $results->())) {
             unless (defined $it) {
                 # Find the an binding iterator for the best pattern from $bgpgs
-                ($it,$bgps) = $self->_find_variable_bindings($bgps);
+                ($it,$bgps) = $self->_find_variable_bindings_($bgps);
 
                 return undef unless $it;
             }
@@ -122,7 +142,7 @@ sub find_variable_bindings {
             # Apply all the bindings to the rest of the bgps;
             my $bgps_prime = $self->_apply_binding($this_binding,$bgps);
 
-            $results = $self->find_variable_bindings($bgps_prime,$bindings);
+            $results = $self->_find_variable_bindings($bgps_prime,$bindings);
         }
         
         $ret;
@@ -142,7 +162,7 @@ sub find_variable_bindings {
 #              patterns are provided or we get zero results
 #              
 #  $rest     - An array ref of patterns not containing the best pattern
-sub _find_variable_bindings {
+sub _find_variable_bindings_ {
     my ($self,$bgps) = @_;
 
     return (undef, undef) unless is_array_ref($bgps) && @$bgps > 0;
@@ -171,9 +191,7 @@ sub _find_variable_bindings {
 
         for (keys %var_map) {
             my $method   = $var_map{$_};
-            my $value    = $triple->$method->as_string;
-            $value =~ s{^<(.*)>$}{$1};
-            $var_map{$_} = $value;
+            $var_map{$_} = $triple->$method;
         }
 
         return {%var_map};
@@ -193,7 +211,11 @@ sub _apply_binding {
     for my $pattern (@$copy) {
         for (qw(subject predicate object)) {
             my $val = $pattern->{$_};
-            $pattern->{$_} = $binding->{$val} if defined($val) && $binding->{$val};
+            if (defined($val) && $binding->{$val}) {
+                my $str_val = $binding->{$val}->as_string;
+                $str_val =~ s{^<(.*)>$}{$1};
+                $pattern->{$_} = $str_val
+            }
         }
         push @new, $pattern;
     }
@@ -201,7 +223,7 @@ sub _apply_binding {
     return \@new;
 }
 
-# Create a pattern which binds optiomally to the graph pattern
+# Create a pattern which binds to the graph pattern
 #
 # Usage:
 #
@@ -299,13 +321,16 @@ sub _parse_triple_pattern {
     my ($subject,$predicate,$object);
 
     $subject   = $triple->subject->as_string;
-    $subject   =~ s{<(.*)>}{$1};
+    $subject   =~ s{^<(.*)>$}{$1};
+    $subject   =~ s{^\((.*)\)$}{?$1};
 
     $predicate = $triple->predicate->as_string;
-    $predicate =~ s{<(.*)>}{$1};
+    $predicate =~ s{^<(.*)>$}{$1};
+    $predicate =~ s{^\((.*)\)$}{?$1};
 
     $object    = $triple->object->as_string;
-    $object    =~ s{<(.*)>}{$1};
+    $object    =~ s{^<(.*)>$}{$1};
+    $object    =~ s{^\((.*)\)$}{?$1};
 
     return {
         subject   => $subject ,
@@ -322,9 +347,10 @@ sub _parse_triple_pattern {
 #   rdf_object    => <name_of_object_variable>
 #   void_uriLookupEndpoint => <endpoint_for_tripple_pattern>
 # }
-sub get_pattern {
+sub get_query_pattern {
     my ($self) = @_;
     my $url      = $self->url;
+
     my $fragment = $self->get_model_and_info($url);
 
     return undef unless defined $fragment;
@@ -354,7 +380,9 @@ sub get_pattern {
     $pattern;
 }
 
-# Given $subject,$predicate,$object return a generator for RDF::Trine::Model
+#----------------------------------------------------------------------------------
+
+# Public method
 sub get_statements {
     my ($self,@triple) = @_;
     my ($subject,$predicate,$object);
@@ -368,7 +396,7 @@ sub get_statements {
         $object    = $triple[0]->{object};
     }
 
-    my $pattern = $self->pattern;
+    my $pattern = $self->query_pattern;
 
     return undef unless defined $pattern;
 
@@ -598,9 +626,26 @@ RDF::LDF - Linked Data Fragments client
 
 =head1 SYNOPSIS
 
-    use RDF::LDF ;
+    use RDF::Trine::Store::LDF;
+    use RDF::Trine::Store;
 
-    my $client = RDF::LDF ->new(url => 'http://fragments.dbpedia.org/2014/en');
+    my $store = RDF::Trine::Store->new_with_config({
+            storetype => 'LDF',
+            url => $url
+    });
+
+    my $it = $store->get_statements();
+
+    while (my $st = $it->next) {
+        # $st is a RDF::Trine::Statement
+        print "$st\n";
+    }
+
+    # Or the low level modules themselves
+
+    use RDF::LDF;
+
+    my $client = RDF::LDF->new(url => 'http://fragments.dbpedia.org/2014/en');
 
     my $iterator = $client->get_statements($subject, $predicate, $object);
 
@@ -608,23 +653,14 @@ RDF::LDF - Linked Data Fragments client
         # $model is a RDF::Trine::Statement
     } 
 
-    my $iterator = $client->get_sparql(<<EOF);
-PREFIX dbpedia: <http://dbpedia.org/resource/>
-SELECT * WHERE { dbpedia:Arthur_Schopenhauer ?predicate ?object . }
-EOF
-
-    while (my $binding = $iterator->()) {
-        # $binding is a hashref of all the bindings in the SPARQL
-    }
 
 =head1 DESCRIPTION
 
 The RDF::LDF  module is a basic implementation of a Linked Data Fragment client. For details see:
 <http://linkeddatafragments.org/>.
 
-=head1 STATUS
-
-THIS IS ALPHA CODE! The implementation is unreliable and the interface is subject to change.
+This a low level module to implement the Linked Data Fragment protocol. You probably want to
+use L<RDF::Trine::Store::LDF>.
 
 =head1 CONFIGURATION
 
@@ -640,34 +676,13 @@ URL to retrieve RDF from.
 
 =over 
 
-=item get_statements($subject,$predicate,$object)
+=item get_statements( $subject, $predicate, $object )
 
 Return an iterator for every RDF::Trine::Statement served by the LDF server.
 
-=item get_sparql($sparql)
+=item get_pattern( $bgp );
 
-Return an iterator for everty binding in the SPARQL query. For now the support is very basic.
-Only select queries are supported without union, sorting or limits with a form like:
-
-
-    PREFIX owl: <http://www.w3.org/2002/07/owl#>
-    PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>
-    PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
-    PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
-    PREFIX foaf: <http://xmlns.com/foaf/0.1/>
-    PREFIX dc: <http://purl.org/dc/elements/1.1/>
-    PREFIX : <http://dbpedia.org/resource/>
-    PREFIX dbpedia2: <http://dbpedia.org/property/>
-    PREFIX dbpedia: <http://dbpedia.org/>
-    PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
-    PREFIX dbo: <http://dbpedia.org/ontology/>
-    SELECT *
-    WHERE {
-      ?car <http://purl.org/dc/terms/subject> <http://dbpedia.org/resource/Category:Luxury_vehicles> .
-      ?car foaf:name ?name .
-      ?car dbo:manufacturer ?man .
-      ?man foaf:name ?manufacturer
-    }
+Returns a stream object of all bindings matching the specified graph pattern.
 
 =back
 
@@ -675,6 +690,13 @@ Only select queries are supported without union, sorting or limits with a form l
 
 Patrick Hochstenbach, C<< patrick.hochstenbach at ugent.be >>
 
+=head1 LICENSE
+
+This program is free software; you can redistribute it and/or modify it under the terms of either: 
+the GNU General Public License as published by the Free Software Foundation; or the Artistic License.
+
+See L<http://dev.perl.org/licenses/> for more information.
+
 =encoding utf8
 
 =cut
\ No newline at end of file
